/*
 * This file is part of Quiz-App.
 *
 * Quiz-App is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Quiz-App is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Quiz-App. If not, see <https://www.gnu.org/licenses/>.
 */
package ch.jaermann.quiz;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Stream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import com.fasterxml.jackson.databind.ObjectMapper;

import ch.jaermann.quiz.model.Quiz;
import lombok.extern.log4j.Log4j2;

@Log4j2
public class Helper {
	
	/**
	 * Liest den Inhalt einer Datei aus dem Classpath.<br>
	 * Das heisst, die Datei muss im src/main|test/resources Ordner liegen.
	 * @param fileName
	 * @return Inhalt der Datei als Text
	 */
	public static StringBuilder readFromRessource(String fileName) {
		
		InputStream inputStream = new Helper().getClass().getClassLoader().getResourceAsStream(fileName);
		BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, StandardCharsets.UTF_8));

		StringBuilder sb = new StringBuilder();
		reader.lines().forEach(line -> sb.append(line).append("\n"));
		return sb;
	}

	public static StringBuilder readFromFile(String fileName) {
		StringBuilder contentBuilder = new StringBuilder();

		try (Stream<String> stream = Files.lines(Paths.get(fileName), StandardCharsets.UTF_8)) {
			stream.forEach(s -> contentBuilder.append(s).append("\n"));
		} catch (IOException e) {
			e.printStackTrace();
		}

		return contentBuilder;
	}
	
	/**
	 * Schreibt den Inhalt in eine Datei
	 * 
	 * @param fileName
	 * @param content
	 */
	public static void writeToFile(String fileName, StringBuilder content) {
		try {
			Files.write(Paths.get(fileName), content.toString().getBytes());
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static void writeToFile(String fileName, Quiz quiz) {
		// Save to file
		ObjectMapper objectMapper = new ObjectMapper();
		try {
			objectMapper.writerWithDefaultPrettyPrinter().writeValue(new File(fileName), quiz);
		} catch (IOException e) {
			log.error("Error writing questions to file: " + fileName, e);
			e.printStackTrace();
		}
	}

	/**
	 * Ersetzt das erste Auftreten von toReplace im gegebenen String txt mit
	 * replacement, ohne dabei RegEx dazu zu benutzen.
	 */
	public static String replaceFirst(String txt, String toReplace, String replacement) {

		int start = txt.indexOf(toReplace);
		
		int end = start + toReplace.length();
		if (start < 0 || end < 0)
			return txt;

		return txt.substring(0, start) + replacement + txt.substring(end);
	}
	
	/**
	 * Ersetzt das erste Auftreten von toReplace im gegebenen String txt mit
	 * replacement, ohne dabei RegEx dazu zu benutzen.
	 */
	public static StringBuilder replaceFirst(StringBuilder txt, String toReplace, String replacement) {
		int start = txt.indexOf(toReplace);
		int end = start + toReplace.length();
		if (start < 0 || end < 0)
			return txt;

		txt.replace(start, end, replacement);
		return txt;
	}
	
	/**
	 * Ersetzt sämtliche Auftreten von toReplace im gegebenen String txt mit
	 * replacement, ohne dabei RegEx dazu zu benutzen.
	 */
	public static StringBuilder replaceAll(StringBuilder txt, String toReplace, String replacement) {
		int start = txt.indexOf(toReplace);
        int end = start + toReplace.length();
        while (start >= 0 && end >= 0) {
            txt.replace(start, end, replacement);
            start = txt.indexOf(toReplace);
            end = start + toReplace.length();
        }
        return txt;
    }
	
	/**
	 * verarbeitet den gegebenen String wie folgt: <br>
	 * [1]          # IntegerArray[1] <br>
	 * [1-3]        # IntegerArray[1,2,3] <br>
	 * [1-3, 5,7]   # IntegerArray[1,2,3,5,7] <br>
	 * [1-3, 5,7 , 12-15]   # IntegerArray[1,2,3,5,7,12,13,14,15] <br>
	 * <br>
	 * [1-3, 5,7 , 12-15:r3]   # aus dem IntegerArray[1,2,3,5,7,12,13,14,15] zufällige, aber verschiedene 3<br>
	 * [1-3, 5,7 , 12-15:r5]   # aus dem IntegerArray[1,2,3,5,7,12,13,14,15] zufällige, aber verschiedene 5<br>
	 * [1-3, 5,7 , 12-15:r]    # das IntegerArray[1,2,3,5,7,12,13,14,15] zufällig, verwürfeln<br>
	 *  
	 * @param args
	 * @param option
	 * @return
	 */
	public static List<Integer> findOption(String option) {
		option = option.replace("[", "");
		option = option.replace("]", "");
		
		String q = option.split(":")[0];
		String r = option.split(":").length > 1 ? option.split(":")[1] : "";
		
		String[] parts = q.split(",");
        ArrayList<Integer> result = new ArrayList<>();
        for (String part : parts) {
            if (part.contains("-")) {
                String[] range = part.split("-");
                int start = Integer.parseInt(range[0].trim());
                int end = Integer.parseInt(range[1].trim());
                for (int j = start; j <= end; j++) {
                    result.add(j);
                }
            } else {
                result.add(Integer.parseInt(part.trim()));
            }
        }
        
        // Parameter nach dem ":" verarbeiten
		if (r.length() > 0) {
			String[] p = r.split(",");
			for (int i = 0; i < p.length; i++) {
				p[i] = p[i].trim();
				
				if (p[i].startsWith("r")) {
					// aus dem IntegerArray zufällige, aber verschiedene herausnehmen
					r = p[i].substring(1);
					
					int count = parseToInt(r, Integer.MAX_VALUE); // Integer.parseInt(r);
					if (count > result.size()) // maximum mögliche überschritten
						count = result.size();
					
					ArrayList<Integer> result2 = new ArrayList<>();
					while (result2.size() < count) {
						int index = (int) (Math.random() * result.size());
						if (!result2.contains(result.get(index))) {
							result2.add(result.get(index));
						}
					}
					result = result2;
				}
			}
		}
        
        return result;
    }
	
	public static int parseToInt(String stringToParse, int defaultValue) {
	    try {
	       return Integer.parseInt(stringToParse);
	    } catch(NumberFormatException ex) {
	       return defaultValue; //Use default value if parsing failed
	    }
	}

	/**
	 * Gibt die aktuelle Zeit im Format "yyyy-MM-dd'T'HH:mm:ss.SSSZ" zurück
	 * @return
	 */
	public static String getCurrentTime() {
		// Get the current date and time with time zone information
		ZonedDateTime currentDateTime = ZonedDateTime.now();

		// Format the date and time to the desired string format
		String formattedDateTime = currentDateTime.format(DateTimeFormatter.ISO_OFFSET_DATE_TIME);
		return formattedDateTime;
	}

	/**
	 * Erstellt eine Zip-Datei aus einer Datei
	 * @param sourceFile
	 * @param zipFileName
	 */
	public static void createZip(String sourceFile, String zipFileName) {

		// TODO die Methode sollte einen Fehler werfen...
		
		try (FileInputStream fis = new FileInputStream(sourceFile);
				FileOutputStream fos = new FileOutputStream(zipFileName);
				ZipOutputStream zipOut = new ZipOutputStream(fos)) {

			ZipEntry zipEntry = new ZipEntry(sourceFile.substring(sourceFile.lastIndexOf("/") + 1));
			zipOut.putNextEntry(zipEntry);

			byte[] bytes = new byte[1024];
			int length;
			while ((length = fis.read(bytes)) >= 0) {
				zipOut.write(bytes, 0, length);
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
		
}
