/*
 * This file is part of Quiz-App.
 *
 * Quiz-App is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Quiz-App is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Quiz-App. If not, see <https://www.gnu.org/licenses/>.
 */
package ch.jaermann.quiz;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.StandardCopyOption;
import java.util.HashMap;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;

import com.fasterxml.jackson.databind.ObjectMapper;

import ch.jaermann.quiz.creater.DocumentCreater;
import ch.jaermann.quiz.creater.DocumentCreaterFactory;
import ch.jaermann.quiz.creater.adoc.DocumentCreaterAdoc;
import ch.jaermann.quiz.model.Question;
import ch.jaermann.quiz.model.Quiz;
import lombok.extern.log4j.Log4j2;


@Log4j2
public class QuizApp {

	private DocumentCreater documentCreater;

	/** url zum Fragepool, */
	private static String questionsPoolUrl = "questions.json";
	
	/** Das gesamte Quiz, beinhaltet sämtliche Fragen für das Quiz. Meistens aus einem Fragepool geladen.*/
	private Quiz quiz;

	/** url zur Vorlage Dokument, */
	private static File quizTemplateUrl = null; // new File("vorlagen/quizTemplate.adoc");
	
	/** Output File */
	private static File outputFileUrl = null; // new File("quiz.adoc");

	public static void main(String[] args) throws IOException {
		
		if (java.lang.management.ManagementFactory.getRuntimeMXBean().getInputArguments().toString().indexOf("-agentlib:jdwp") > 0) {

			System.out.println("Debugging aktiviert");
			
			args = new String[] { "-g", "-nn", "-q", "questions.json", "-t", "quizTemplate.adoc" };
			args = new String[] { "-g", "-nn", "-q", "2020f.json", "-t", "quizTemplate.adoc" };
			args = new String[] { "-g", "-nn", "-q", "2020f.json", "-t", "quizM183LB2_F.adoc" };
			args = new String[] { "-o", "LB2_F.adoc", "-g", "-nn", "-q", "2020f.json", "-t", "quizM183LB2_F.adoc" };
			args = new String[] { "" };
			args = new String[] { "-g", "-ot", "txt", "-t", "quizTemplate.txt", "-o", "quiz.txt" }; // TXT
			args = new String[] { "-g", "-ot", "adoc", "-t", "quizTemplate.adoc", "-o", "quiz.adoc" }; // ADOC
			args = new String[] { "-g"}; // ADOC
			args = "-g -q questions.json -n -nn".split(" "); // new ID's
			args = "-g -ot sm".split(" "); // SM
			args = "-g -ot libre".split(" "); // Libreoffice
			args = "-g -ot txt".split(" "); // TXT
			args = "-g -ot txt".split("-g  "); // 
		}
		
		if (args.length == 0) {
			args = new String[] { "-h" };
		}
		new QuizApp(args);
	}

	public QuizApp(String[] args) throws IOException {

		Options options = new Options();

		Option help = new Option("h", "help", false, "show help and exit");
		help.setRequired(false);
		options.addOption(help);

		Option questionsPool = new Option("q", "questions", true, "questions pool file path");
		questionsPool.setRequired(false);
		options.addOption(questionsPool);

		Option input = new Option("i", "input", true, "input file url");
		input.setRequired(false);
		options.addOption(input);

		Option quiztemplate = new Option("t", "quizTemplate", true, "quizTemplate file url");
		quiztemplate.setRequired(false);
		options.addOption(quiztemplate);

		Option generate = new Option("g", "generate", false, "generate examplefiles");
		generate.setRequired(false);
		options.addOption(generate);
		
		Option questionsID = new Option("n", "questionsID", false, "generates a unique ID for all elements without ID");
		questionsID.setRequired(false);
		options.addOption(questionsID);
		
		Option questionsID2 = new Option("nn", "questionsNewIDs", false, "generates new IDs for all elements, start with 1");
		questionsID2.setRequired(false);
		options.addOption(questionsID2);
		
		Option outputFile = new Option("o", "outputFileUrl", true, "outputFileUrl");
		outputFile.setRequired(false);
		options.addOption(outputFile);
		
		Option outputType = new Option("ot", "outputType", true, "outputFile type, [adoc | sm | txt | libre ] default adoc");
		outputType.setRequired(false);
		options.addOption(outputType);

		CommandLineParser parser = new DefaultParser();
		HelpFormatter formatter = new HelpFormatter();
		CommandLine cmd;
		

		try {
			cmd = parser.parse(options, args);
		} catch (org.apache.commons.cli.ParseException e) {
			System.out.println(e.getMessage());
			formatter.printHelp("quizz-app", options);

			System.exit(-1);
			return;
		}

		if (cmd.hasOption("help")) {
			System.out.println(""+QuizApp.class.getPackage().getImplementationTitle() + " " + QuizApp.class.getPackage().getImplementationVersion());
			formatter.printHelp("quizz-app", options);
			System.exit(0);
		}

		if (cmd.hasOption("g")) {
			generateExampleFiles();
		}
		
		if (cmd.hasOption("o") && cmd.getOptionValue("o") != null && !cmd.getOptionValue("o").isBlank()) {
			outputFileUrl = new File(cmd.getOptionValue("o"));
			log.info("using outputFileUrl " + outputFileUrl);
		} else {
			// System.out.println("No outputFileUrl specified, using default " + outputFileUrl);
			log.info("No outputFileUrl specified, using default " + outputFileUrl);
		}

		if (cmd.hasOption("q") && !cmd.getOptionValue("q").isBlank()) {
			questionsPoolUrl = cmd.getOptionValue("q");
			log.info("using questions pool file " + questionsPoolUrl);
		} else {
			System.out.println("No questions pool file specified, using default " + questionsPoolUrl);
			log.info("No questions pool file specified, using default " + questionsPoolUrl);
		}
		quiz = readQuestions(questionsPoolUrl);
		
		if (cmd.hasOption("n")) {
			quiz = generateIDs(quiz);
			Helper.writeToFile(questionsPoolUrl, quiz);
		}
		if (cmd.hasOption("nn")) {
			quiz = generateNewIDs(quiz);
			Helper.writeToFile(questionsPoolUrl, quiz);
		}
		
		if (cmd.hasOption("ot")) {
			String ot = cmd.getOptionValue("ot").toLowerCase().trim();			
			documentCreater = DocumentCreaterFactory.create(ot);			
		}else {
			System.out.println("No outputType specified, using outputType adoc");
			log.info("No outputType specified, using outputType adoc");
			documentCreater = new DocumentCreaterAdoc();
		}

		if (cmd.hasOption("t") && !cmd.getOptionValue("t").isBlank()) {
			quizTemplateUrl = new File(cmd.getOptionValue("t"));
			log.info("using quizTemplate file " + quizTemplateUrl);
		} else {
			System.out.println("No quizTemplate file specified, using " + quizTemplateUrl);
			log.info("No quizTemplate file specified, using default " + quizTemplateUrl);
		}

		// übriggebliebene Argumente ausgeben
		for (String s : cmd.getArgs()) {
			if(s.length() > 0 ) {
				System.out.println("Leerer Parameter" + s);
				log.info(s);
			}
		}
		
		documentCreater.create(quiz, quizTemplateUrl, outputFileUrl);	
	}

	/**
	 * Generiert und überschreibt für jedes Element eine neue ID beginnend bei 1
	 * @param original
	 * @return
	 */
	public static Quiz generateNewIDs(Quiz original) {

		int nextID = 1;

		for (Question question : original.getQuestions()) {

			question.setId(nextID++);

		}
		
		return original;
	}
	
	/**
	 * Generiert für jedes Element welches eine ID==0 hat eine neue ID aufsteigend beginnend bei 1.
	 * @param original
	 * @return
	 */
	public static Quiz generateIDs(Quiz original) {
		
		HashMap<String, Integer> idMap = new HashMap<>();
		// ID's sammeln
		for (Question question : original.getQuestions()) {
			if (question.getId() != 0) {
				idMap.put("" + question.getId(), question.getId());
			}
		}		
		
		int nextID = 1;
		
		for (Question question : original.getQuestions()) {
			
			if (question.getId() == 0) {
				
				// ID noch nicht vergeben?
				while (idMap.containsKey("" + nextID)) {
					nextID++;
				}
				
				question.setId(nextID);
				idMap.put(""+question.getId(), question.getId());
				continue;
			}			
		}
		return original;
	}

	/**
	 * Kopiert die Beispieldateien aus den Resources in das aktuelle Verzeichnis
	 */
	private void generateExampleFiles() {

		// copy questions.json from resources to root
		File questions = new File("questions.json");
		if (!questions.exists()) {
			try (InputStream questionsInput = getClass().getResourceAsStream("/vorlagen/questions.json")) {
				Files.copy(questionsInput, Path.of("questions.json"), StandardCopyOption.REPLACE_EXISTING);
			} catch (IOException e) {
				log.error("Error copying questions.json from resources", e);
				e.printStackTrace();
			}
		}

		// copy /vorlagen/quizTemplate.adoc from resources to root
		File quizTemplateAdoc = new File("quizTemplate.adoc");
		if (!quizTemplateAdoc.exists()) {
			try (InputStream quizTemplateInput = getClass().getResourceAsStream("/vorlagen/quizTemplate.adoc")) {
				Files.copy(quizTemplateInput, Path.of("quizTemplate.adoc"), StandardCopyOption.REPLACE_EXISTING);
			} catch (IOException e) {
				log.error("Error copying quizTemplate.adoc from resources", e);
				e.printStackTrace();
			}
		}
		
		// copy /vorlagen/quizTemplate.txt from resources to root
		File quizTemplateTxt = new File("quizTemplate.txt");
		if (!quizTemplateTxt.exists()) {
			try (InputStream quizTemplateInput = getClass().getResourceAsStream("/vorlagen/quizTemplate.txt")) {
				Files.copy(quizTemplateInput, Path.of("quizTemplate.txt"), StandardCopyOption.REPLACE_EXISTING);
			} catch (IOException e) {
				log.error("Error copying quizTemplate.txt from resources", e);
				e.printStackTrace();
			}
		}
		
		// copy /vorlagen/quizTemplate.txt from resources to root
		File quizTemplateOdt = new File("quizTemplate.odt");
		if (!quizTemplateOdt.exists()) {
			try (InputStream quizTemplateInput = getClass().getResourceAsStream("/vorlagen/quizTemplate.odt")) {
				Files.copy(quizTemplateInput, Path.of("quizTemplate.odt"), StandardCopyOption.REPLACE_EXISTING);
			} catch (IOException e) {
				log.error("Error copying quizTemplate.odt from resources", e);
				e.printStackTrace();
			}
		}
		
		return;
	}

	private Quiz readQuestions(String questionsPoolUrl) {
		ObjectMapper objectMapper = new ObjectMapper();
		try {
			Quiz quiz = objectMapper.readValue(new File(questionsPoolUrl), Quiz.class);
			return quiz;
		} catch (IOException e) {
			log.error("Error reading questions from file: " + questionsPoolUrl, e);
			e.printStackTrace();
		}
		return null;
	}
}
