/*
 * This file is part of Quiz-App.
 *
 * Quiz-App is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Quiz-App is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Quiz-App. If not, see <https://www.gnu.org/licenses/>.
 */
package ch.jaermann.quiz.creater.adoc;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.nio.file.Files;
import java.util.Iterator;
import java.util.List;

import com.fasterxml.jackson.core.exc.StreamReadException;
import com.fasterxml.jackson.databind.DatabindException;
import com.fasterxml.jackson.databind.ObjectMapper;

import ch.jaermann.quiz.Helper;
import ch.jaermann.quiz.controller.QuizController;
import ch.jaermann.quiz.creater.DocumentCreater;
import ch.jaermann.quiz.creater.adoc.controller.QuestionFactory;
import ch.jaermann.quiz.creater.txt.DocumentCreaterTxt;
import ch.jaermann.quiz.model.Question;
import ch.jaermann.quiz.model.Quiz;
import lombok.extern.log4j.Log4j2;

@Log4j2
public class DocumentCreaterAdoc extends DocumentCreater{
	/** Fragen für das Quiz, */
	public Quiz quiz;

	public static void main(String[] args) throws StreamReadException, DatabindException, IOException {
		URL resource = new DocumentCreaterTxt().getClass().getClassLoader().getResource("vorlagen/questions.json");
		ObjectMapper objectMapper = new ObjectMapper();

		Quiz quiz = objectMapper.readValue(new File(resource.getFile()), Quiz.class);

		new DocumentCreaterAdoc().create(quiz, null, null);
	}

	@Override
	public boolean create(Quiz quiz, File template, File outputFile) throws IOException {
	
		QuizController quizController = new QuizController(quiz);
		StringBuilder quizTemplateContent = null;
		int points = 0;
		
		if (template == null || !template.canRead()) {
			System.out.println("template is empty -> using default template quizTemplate.adoc");
			log.info("template is empty -> using default template quizTemplate.adoc");
			quizTemplateContent = Helper.readFromRessource("vorlagen/quizTemplate.adoc");
		}else {
			quizTemplateContent = new StringBuilder( Files.readString(template.toPath()));
		}
		
		if (outputFile == null) {
			System.out.println("outputFile is empty -> using default quiz.adoc");
			log.info("outputFile is empty -> using default quiz.adoc");
			outputFile = new File("quiz.adoc");
		}
		
		if (quizTemplateContent == null || quizTemplateContent.length() == 0) {
			System.out.println("quizTemplateContent is empty -> using default template");
			log.info("quizTemplateContent is empty -> using default template");
			quizTemplateContent = Helper.readFromRessource("vorlagen/quizTemplate.adoc");
		}

		if (quizTemplateContent.indexOf("${questions}", 0) >= 0) {
			StringBuilder s = expandQuestions(quiz);
			quizTemplateContent = Helper.replaceAll(quizTemplateContent, "${questions}", s.toString());
			points+=quizController.calculateTotalPoints();
		}
		
		String toReplace = "${questionTitel}[";
		int start = quizTemplateContent.indexOf(toReplace);
		while (start >= 0) {
			// Welcher?
			int end = quizTemplateContent.indexOf("]", start + toReplace.length());
			String option = quizTemplateContent.substring(start + toReplace.length(), end);

			List<Integer> list = Helper.findOption(option);
			Quiz q = quizController.getQuestionsWithId(list);
			// create Fragen in der Liste
			StringBuilder s = expandQuestionTitel(q);
			
			quizTemplateContent = quizTemplateContent.replace(start, end+1, s.toString());
			start = quizTemplateContent.indexOf(toReplace);
		}
		
		toReplace = "${questionText}[";
		start = quizTemplateContent.indexOf(toReplace);
		while (start >= 0) {
			// Welcher?
			int end = quizTemplateContent.indexOf("]", start + toReplace.length());
			String option = quizTemplateContent.substring(start + toReplace.length(), end);
			
			List<Integer> list = Helper.findOption(option);
			Quiz q = quizController.getQuestionsWithId(list);
			// create Fragen in der Liste
			StringBuilder s = expandQuestionText(q);
			
			quizTemplateContent = quizTemplateContent.replace(start, end+1, s.toString());
			start = quizTemplateContent.indexOf(toReplace);
		}
		
		toReplace = "${question}[";
		start = quizTemplateContent.indexOf(toReplace);
		while (start >= 0) {
			// Welcher?
			int end = quizTemplateContent.indexOf("]", start + toReplace.length());
			String option = quizTemplateContent.substring(start + toReplace.length(), end);
			
			List<Integer> list = Helper.findOption(option);
			Quiz q = quizController.getQuestionsWithId(list);
			// create Fragen in der Liste
			StringBuilder s = expandQuestions(q);
			
			quizTemplateContent = quizTemplateContent.replace(start, end+1, s.toString());
			start = quizTemplateContent.indexOf(toReplace);
			
			points+=quizController.calculateTotalPoints(q);
		}

		quizTemplateContent = Helper.replaceAll(quizTemplateContent, "${points}", ""+points);
		
		// write to file
		Helper.writeToFile(outputFile.toString(), quizTemplateContent);
		
		return true;
	}
	
	private StringBuilder expandQuestionTitel(Quiz questions) {
		
		StringBuilder q = new StringBuilder();

		for (Iterator<Question> iterator = questions.getQuestions().iterator(); iterator.hasNext();) {
			Question question =  iterator.next();
			
			q.append(question.getTitel());
			q.append("\n");
			
		}
		return q;
	}

	private StringBuilder expandQuestionText(Quiz questions) {
		
		StringBuilder q = new StringBuilder();
		
		for (Iterator<Question> iterator = questions.getQuestions().iterator(); iterator.hasNext();) {
			Question question =  iterator.next();
			
			q.append(question.getQuestionText());
			q.append("\n");
			
		}
		return q;
	}

	private StringBuilder expandQuestion(Question question) {
		
		StringBuilder s=QuestionFactory.createQuestion(question).toAdoc();
		return s;
	}
	
	/**
	 * 
	 * 
	 * @param questions
	 * @param questionList
	 * @return
	 */
	private StringBuilder expandQuestions(Quiz questions) {

		StringBuilder q = new StringBuilder();

		for (Iterator<Question> iterator = questions.getQuestions().iterator(); iterator.hasNext();) {
			Question question =  iterator.next();
			
			q.append(expandQuestion(question));
			q.append("\n");
		}

		return q;
	}
	
}