/*
 * This file is part of Quiz-App.
 *
 * Quiz-App is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Quiz-App is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Quiz-App. If not, see <https://www.gnu.org/licenses/>.
 */
package ch.jaermann.quiz.creater.libreoffice;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.Iterator;
import java.util.List;

import org.odftoolkit.odfdom.doc.OdfDocument;
import org.odftoolkit.odfdom.doc.OdfTextDocument;
import org.odftoolkit.odfdom.dom.element.text.TextPElement;
import org.odftoolkit.odfdom.incubator.search.TextNavigation;
import org.odftoolkit.odfdom.incubator.search.TextSelection;
import org.xml.sax.SAXException;

import com.fasterxml.jackson.core.exc.StreamReadException;
import com.fasterxml.jackson.databind.DatabindException;
import com.fasterxml.jackson.databind.ObjectMapper;

import ch.jaermann.quiz.Helper;
import ch.jaermann.quiz.controller.QuizController;
import ch.jaermann.quiz.creater.DocumentCreater;
import ch.jaermann.quiz.creater.txt.DocumentCreaterTxt;
import ch.jaermann.quiz.model.Question;
import ch.jaermann.quiz.model.Quiz;
import lombok.extern.log4j.Log4j2;

@Log4j2
public class DocumentCreaterLibreoffice extends DocumentCreater {

	public static void main(String[] args) throws StreamReadException, DatabindException, IOException {
		URL resource = new DocumentCreaterTxt().getClass().getClassLoader().getResource("vorlagen/questions.json");
		ObjectMapper objectMapper = new ObjectMapper();

		Quiz quiz = objectMapper.readValue(new File(resource.getFile()), Quiz.class);

		boolean b = new DocumentCreaterLibreoffice().create(quiz, new File("src/main/resources/vorlagen/quizTemplate.odt"), new File("quiz.odt"));
		System.out.println(b);
	}
		
	@Override
	public boolean create(Quiz quiz, File template, File outputFile) {
		
		QuizController quizController = new QuizController(quiz);
		int points = 0;
		
		try {
			// Load an existing text document from local file
			OdfDocument doc = null;
			if (template == null || template.canRead() == false) {
				System.out.println("template is empty -> using default template vorlagen/quizTemplate.odt");
				log.info("template is empty -> using default template vorlagen/quizTemplate.odt");
				doc = OdfDocument.loadDocument(this.getClass().getClassLoader().getResourceAsStream("vorlagen/quizTemplate.odt"));
			}else {
				doc = OdfDocument.loadDocument(template);
			}
			
			if (outputFile == null) {
				System.out.println("outputFile is empty -> using default quiz.odt");
				log.info("outputFile is empty -> using default quiz.odt");
				outputFile = new File("quiz.odt");
			}
			
			// Suchtext definieren
			String searchText = "\\$\\{questions\\}"; // findet alle ${questions}
			TextNavigation search = new TextNavigation(searchText, (OdfTextDocument) doc);
			// Navigate through the document....
			while (search.hasNext()) {
				TextSelection selection = search.next();
				
				expandQuestions(quiz, selection, doc);
				
				selection.replaceWith(""); // löschen von ${questions}...

				points+=quizController.calculateTotalPoints(quiz);
			}
			doc.save(outputFile);
			
			// Suchtext definieren
			searchText = "\\$\\{question\\}\\[.*\\]"; // findet alle ${question}[<id>-<id>:r<anzahl>]
			search = new TextNavigation(searchText, (OdfTextDocument) doc);
			// Navigate through the document....
			while (search.hasNext()) {
				TextSelection selection = search.next();
				
				String option = selection.getText();
				option=option.substring("${question}".length());
				
				List<Integer> list = Helper.findOption(option);
				Quiz q = quizController.getQuestionsWithId(list);
				
				expandQuestions(q, selection, doc);
				
				selection.replaceWith(""); // löschen von ${question}...
				
				points+=quizController.calculateTotalPoints(q);
			}
			doc.save(outputFile);
			
			// Suchtext definieren
			searchText = "\\$\\{questionTitel\\}\\[.*\\]"; // findet alle ${questionTitel}[<id>-<id>:r<anzahl>]
			search = new TextNavigation(searchText, (OdfTextDocument) doc);
			// Navigate through the document....
			while (search.hasNext()) {
				TextSelection selection = search.next();
				
				String option = selection.getText();
				option=option.substring("${questionTitel}".length());
				
				List<Integer> list = Helper.findOption(option);
				Quiz q = quizController.getQuestionsWithId(list);
				
				expandQuestionsTitel(q, selection, doc);
				
				selection.replaceWith(""); // löschen von ${question}...
				
				points+=quizController.calculateTotalPoints(q);
			}
			doc.save(outputFile);
			
			// Suchtext definieren
			searchText = "\\$\\{questionText\\}\\[.*\\]"; // findet alle ${questionText}[<id>-<id>:r<anzahl>]
			search = new TextNavigation(searchText, (OdfTextDocument) doc);
			// Navigate through the document....
			while (search.hasNext()) {
				TextSelection selection = search.next();
				
				String option = selection.getText();
				option=option.substring("${questionText}".length());
				
				List<Integer> list = Helper.findOption(option);
				Quiz q = quizController.getQuestionsWithId(list);
				
				expandQuestionsText(q, selection, doc);
				
				selection.replaceWith(""); // löschen von ${question}...
				
				points+=quizController.calculateTotalPoints(q);
			}
			doc.save(outputFile);
			
			// Suchtext definieren
			searchText = "\\$\\{points\\}"; // findet alle ${points}
			search = new TextNavigation(searchText, (OdfTextDocument) doc);
			// Navigate through the document....
			while (search.hasNext()) {
				TextSelection selection = search.next();
				
				String option = selection.getText();
				option=option.substring("${points}".length());
				
				selection.replaceWith(""+points); // einsetzen der maximale Anzahl Punkte ...
			}
			doc.save(outputFile);
			

			// Speichern des Dokuments unter einem neuen Namen oder überschreiben Sie das Original
			doc.save(outputFile);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return true;
	}
	
	
	private TextPElement expandQuestions(Quiz questions, TextSelection selection, OdfDocument doc) throws SAXException, IOException {
		
		TextPElement t = null;
		for (Iterator<Question> iterator = questions.getQuestions().iterator(); iterator.hasNext();) {
			Question question = iterator.next();

			t = expandQuestion(question, selection, doc);
		}
				
		return t;
	}
	
	private TextPElement expandQuestionsTitel(Quiz questions, TextSelection selection, OdfDocument doc) throws SAXException, IOException {
		
		TextPElement t = null;
		for (Iterator<Question> iterator = questions.getQuestions().iterator(); iterator.hasNext();) {
			Question question = iterator.next();
			
			t = expandQuestionTitel(question, selection, doc);
		}
		
		return t;
	}
	
	private TextPElement expandQuestionsText(Quiz questions, TextSelection selection, OdfDocument doc) throws SAXException, IOException {
		
		TextPElement t = null;
		for (Iterator<Question> iterator = questions.getQuestions().iterator(); iterator.hasNext();) {
			Question question = iterator.next();
			
			t = expandQuestionText(question, selection, doc);
		}
		
		return t;
	}
	
	private TextPElement expandQuestionText(Question question, TextSelection selection, OdfDocument doc) throws SAXException, IOException {
		
		return QuestionFactory.createQuestion(question, doc).createText(selection);
	}
	
	private TextPElement expandQuestionTitel(Question question, TextSelection selection, OdfDocument doc) throws SAXException, IOException {
		
		return QuestionFactory.createQuestion(question, doc).createTitel(selection);
	}
	
	private TextPElement expandQuestion(Question question, TextSelection selection, OdfDocument doc) throws SAXException, IOException {
		
		return QuestionFactory.createQuestion(question, doc).create(selection);
	}

}
