/*
 * This file is part of Quiz-App.
 *
 * Quiz-App is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Quiz-App is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Quiz-App. If not, see <https://www.gnu.org/licenses/>.
 */
package ch.jaermann.quiz.creater.smartlearn;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.nio.file.Files;
import java.security.SecureRandom;
import java.util.concurrent.atomic.AtomicInteger;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.exc.StreamReadException;
import com.fasterxml.jackson.databind.DatabindException;
import com.fasterxml.jackson.databind.ObjectMapper;

import ch.jaermann.quiz.Helper;
import ch.jaermann.quiz.controller.QuizController;
import ch.jaermann.quiz.creater.DocumentCreater;
import ch.jaermann.quiz.creater.smartlearn.controller.QuestionBuilder;
import ch.jaermann.quiz.creater.smartlearn.controller.QuestionBuilderFactory;
import ch.jaermann.quiz.creater.txt.DocumentCreaterTxt;
import ch.jaermann.quiz.model.Quiz;
import ch.jaermann.quiz.smartlearn.model.Assignment;
import ch.jaermann.quiz.smartlearn.model.Root;
import ch.jaermann.quiz.smartlearn.model.TaskBlock;
import lombok.Data;
import lombok.extern.log4j.Log4j2;

@Data
@Log4j2
public class DocumentCreaterSmartLearn extends DocumentCreater{
	
	/** Fragen für das Quiz, */
	public Quiz quiz;
	
	/** für die vergabe von eindeutigen Import id's für SM*/
	private AtomicInteger smImportId = new AtomicInteger(1);
	
    /** Characters used for random code generation. */
	private static final String CHARACTERS = "abcdefghijklmnopqrstuvwxyz0123456789";
	/** Length of the random code. */
    private static final int CODE_LENGTH = 11;
    /** Secure random number generator. */
    private static final SecureRandom random = new SecureRandom();
    
	/**
	 * Generates a random code.<br>
	 * SM likes to have a random code for the import.<br>
	 *
	 * @return the random code
	 */
    public static String generateRandomCode() {
        StringBuilder code = new StringBuilder(CODE_LENGTH);
        for (int i = 0; i < CODE_LENGTH; i++) {
            int index = random.nextInt(CHARACTERS.length());
            code.append(CHARACTERS.charAt(index));
        }
        return code.toString();
    }
	

	public static void main(String[] args) throws StreamReadException, DatabindException, IOException {
		URL resource = new DocumentCreaterTxt().getClass().getClassLoader().getResource("vorlagen/questions.json");
		ObjectMapper objectMapper = new ObjectMapper();

		Quiz quiz = objectMapper.readValue(new File(resource.getFile()), Quiz.class);

		new DocumentCreaterSmartLearn().create(quiz, null, null);
	}


	@Override
	public boolean create(Quiz quiz, File template, File outputFile) throws IOException {
		
		QuizController quizController = new QuizController(quiz);
		StringBuilder quizTemplateContent = null;
		int points = 0;
		
		if (template == null || !template.canRead()) {
			System.out.println("template is empty -> using default template quizTemplate.adoc");
			log.info("template is empty -> using default template");
			quizTemplateContent = Helper.readFromRessource("vorlagen/quizTemplate.adoc");
		}else {
			quizTemplateContent = new StringBuilder( Files.readString(template.toPath()));
		}
		
		if (outputFile == null) {
			System.out.println("outputFile is empty -> using default importMeToSM.zip");
			log.info("outputFile is empty -> using default importMeToSM.zip");
			outputFile = new File("importMeToSM.zip");
		}

		// durch die Fragen gehen und die Fragen in die SmartLearn Struktur einfügen
		Root smRoot = new Root();
		smRoot.setVersion("v1.0");
		smRoot.setExportedAt(Helper.getCurrentTime()); // create String like "2024-06-22T08:27:23+02:00" from the current time
		
		Assignment smAssignment = new Assignment(); // Kapitel Level, das Assigment enthällt ein einzelnes Kapitel
		smRoot.addAssignments(smAssignment);
		smAssignment.setTitle("Kapitel 1");
		
		// durch die Fragen gehen und die Fragen in die SmartLearn Struktur einfügen
		QuestionBuilderFactory factory = new QuestionBuilderFactory(this, quiz);
		for(ch.jaermann.quiz.model.Question q : quizController.getQuiz().getQuestions()) {
			
			QuestionBuilder questionBuilder = factory.create(q);
			TaskBlock smTaskBlock = questionBuilder.create(q);
			smAssignment.addBlock(smTaskBlock);			
		}
		
		String jsonString = "";
		try {
			ObjectMapper objectMapper = new ObjectMapper();
			jsonString = objectMapper.writerWithDefaultPrettyPrinter().writeValueAsString(smRoot);
			
			// TODO: das ist nicht cool dies hier zu machen.... sollte ausserhlab der klasse sein
			Helper.writeToFile("data.json", new StringBuilder(jsonString));
			// erstelle ein zip aus dem File
			Helper.createZip("data.json", outputFile.toString());
			
			return true;
		} catch (JsonProcessingException e) {
			log.error("Error converting to JSON", e);
			System.out.println("Error converting to JSON");
			e.printStackTrace();
			throw new RuntimeException("Error converting to JSON");
		}
	}

}


